#from netCDF4 import Dataset
import numpy as np
import os
from pylab import *
import matplotlib.gridspec as gridspec
import scipy.io as sio
from scipy import optimize
from netCDF4 import Dataset

#% parameters
fontsize=16;
lw=3;
msize=10;

figwidth=14;
figheight=6;

matplotlib.rcParams.update({'font.size': fontsize})
mpl.rcParams['lines.linewidth'] = lw
mpl.rcParams['lines.markersize'] = msize
mpl.rcParams['lines.markeredgewidth'] = 2
#
# Change subplot sizes
#
gs = gridspec.GridSpec(1, 4)
gs.update(left=0.1, right=0.95, top=0.95,
      bottom=0.125, wspace=0.05, hspace=0.125)

gs2 = gridspec.GridSpec(1, 3)
gs2.update(left=0.1, right=0.95, top=0.95,
      bottom=0.145, wspace=0.125, hspace=0.125)

# 
figures2plot = [1,1,1,1,1,1,1,1]

data = Dataset('PRF2021_ChauchatEtAl.nc', "r", format="NETCDF4")

# Common parameters
wfall = data.groups['parameters']['wfall'][:]
d50 = data.groups['parameters']['d50'][:]
rho_f = data.groups['parameters']['rho_f'][:]
rho_p = data.groups['parameters']['rho_p'][:]
u_star_shear = data.groups['parameters']['u_star_shear'][:]

# load Revil-Baudard et al (2015, 2016) and LES results
H_LES = data.groups['LES']['H_LES'][:]
hb_LES = data.groups['LES']['hb_LES'][:]
H_exp = data.groups['Experiment']['H_exp'][:]
Z_LES = data.groups['LES']['Z_LES'][:]
Z_exp = data.groups['Experiment']['Z_exp'][:]
U_LES = data.groups['LES']['U_LES'][:]
U_exp = data.groups['Experiment']['U_exp'][:]
phi_LES = data.groups['LES']['phi_LES'][:]
phi_exp = data.groups['Experiment']['phi_exp'][:]
wrms_LES = data.groups['LES']['wrms_LES'][:]
wrms_exp = data.groups['Experiment']['wrms_exp'][:]
uw_LES  = data.groups['LES']['uw_LES'][:]
uw_exp  = data.groups['Experiment']['uw_exp'][:]
lm_LES  = data.groups['LES']['lm_LES'][:]
lm_exp  = data.groups['Experiment']['lm_exp'][:]
nut_LES = data.groups['LES']['nut_LES'][:]
nut_exp = data.groups['Experiment']['nut_exp'][:]
Sc_LES  = data.groups['LES']['Sc_LES'][:]
Sc_exp  = data.groups['Experiment']['Sc_exp'][:]
gradphi_LES = data.groups['LES']['gradphi_LES'][:]
gradphi_exp = data.groups['Experiment']['gradphi_exp'][:]
uphi_LES = data.groups['LES']['uphi_LES'][:]
uphi_exp = data.groups['Experiment']['uphi_exp'][:]
wphi_LES = data.groups['LES']['wphi_LES'][:]
wphi_exp = data.groups['Experiment']['wphi_exp'][:]

# load Shen and Lehmin (1999) data
dataShen = Dataset('ShenDataPRF.nc', "r", format="NETCDF4")

rho_sand = dataShen.groups['Shen']['rho_sand'][0]
dSL99run1  = dataShen.groups['Shen']['dSL99run1'][0]
dSL99run2  = dataShen.groups['Shen']['dSL99run2'][0]
HSL99run1  = dataShen.groups['Shen']['HSL99run1'][0]
HSL99run2  = dataShen.groups['Shen']['HSL99run2'][0]
WsSL99run1  = dataShen.groups['Shen']['WsSL99run1'][0]
WsSL99run2  = dataShen.groups['Shen']['WsSL99run2'][0]
ufricSL99run1  = dataShen.groups['Shen']['ufricSL99run1'][0]
ufricSL99run2  = dataShen.groups['Shen']['ufricSL99run2'][0]

ZSL99run1 = dataShen.groups['Shen']['ZSL99run1'][:]
ZSL99run2 = dataShen.groups['Shen']['ZSL99run2'][:]
USL99run1 = dataShen.groups['Shen']['USL99run1'][:]
USL99run2 = dataShen.groups['Shen']['USL99run2'][:]
phiSL99run1  = dataShen.groups['Shen']['phiSL99run1'][:]
phiSL99run2  = dataShen.groups['Shen']['phiSL99run2'][:]
CpWpSL99run1  = dataShen.groups['Shen']['CpWpSL99run1'][:]
CpWpSL99run2  = dataShen.groups['Shen']['CpWpSL99run2'][:]
uwSL99run1 = dataShen.groups['Shen']['uwSL99run1'][:]
uwSL99run2 = dataShen.groups['Shen']['uwSL99run2'][:]
WrmsSL99run1 = dataShen.groups['Shen']['WrmsSL99run1'][:]
WrmsSL99run2 = dataShen.groups['Shen']['WrmsSL99run2'][:]

lm_SL99run1 = dataShen.groups['Shen']['lm_SL99run1'][:]
lm_SL99run2 = dataShen.groups['Shen']['lm_SL99run2'][:] 
zdSL99run1 = dataShen.groups['Shen']['zdSL99run1'][:]
zdSL99run2 = dataShen.groups['Shen']['zdSL99run2'][:]
ScFluxSL99run1 = dataShen.groups['Shen']['ScFluxSL99run1'][:]
ScFluxSL99run2 = dataShen.groups['Shen']['ScFluxSL99run2'][:]
ScSL99run1 = dataShen.groups['Shen']['ScSL99run1'][:]
ScSL99run2 = dataShen.groups['Shen']['ScSL99run2'][:]
CwSL99run1 = dataShen.groups['Shen']['CwSL99run1'][:]
CwSL99run2 = dataShen.groups['Shen']['CwSL99run2'][:]

#########

gradUExp = np.zeros(np.size(U_exp))
gradUExp[:-1] = np.diff(U_exp)/np.diff(Z_exp)

gradULES = np.zeros(np.size(U_LES))
gradULES[:-1] = np.diff(U_LES)/np.diff(Z_LES)

lm_LES = np.sqrt(np.abs(uw_LES))/(np.abs(gradULES))
lm_LES[U_LES*H_LES/1e-6<=1e3]=0

nut_LES = np.abs(uw_LES)/(np.abs(gradULES))
nut_LES[U_LES*H_LES/1e-6<=1e3]=0

#Compute mixing length parameters : kappa and zd
range_expLm = np.where(np.logical_and(Z_exp/H_exp<=0.275,Z_exp/H_exp>=0.05))
# define our (line) fitting function
fitfunc = lambda p, x: p[0]*(x-p[1])
errfunc = lambda p, x, y: y - fitfunc(p, x)
# 
pinit = [0.41, 0.0]
pfinal, success = optimize.leastsq(errfunc, pinit[:], args=(Z_exp[range_expLm], lm_exp[range_expLm]))
#
kappa_exp=pfinal[0]
zdexp=pfinal[1]

print ('Kappa=',pfinal[0],' and z_d=',pfinal[1])

Nexp=np.size(Z_exp)

# reference elevation for Rouse
zr=0.1
zmaxAve=0.25

Z_exp2=(Z_exp/H_exp)-zr
range_exp = np.where(np.logical_and(Z_exp/H_exp<=zmaxAve,Z_exp/H_exp>=zr))    

Scexp = np.nanmean((nut_exp[range_exp]*np.abs(gradphi_exp[range_exp]))/(phi_exp[range_exp]*wfall))
print ('Scexp=',Scexp)
ScexpN = np.nanmean((nut_exp[range_exp]*np.abs(gradphi_exp[range_exp]))/wphi_exp[range_exp])
print ('ScexpN=',ScexpN)
CwexpN = np.nanmean(wphi_exp[range_exp]/(phi_exp[range_exp]*wfall))
print ('CwexpN=',CwexpN)

range_LES = np.where(np.logical_and((Z_LES-hb_LES)/H_LES<=zmaxAve,(Z_LES-hb_LES)/H_LES>=zr))    


gradUrun1 = np.zeros(np.size(ZSL99run1))
gradUrun1[:-1] = np.diff(USL99run1)/(np.diff(ZSL99run1)*HSL99run1)

gradUrun2 = np.zeros(np.size(ZSL99run2))
gradUrun2[:-1] = np.diff(USL99run2)/(np.diff(ZSL99run2)*HSL99run2)

rangeSL99run1 = np.where(np.logical_and(ZSL99run1<=zmaxAve,ZSL99run1>=zr))    
rangeSL99run2 = np.where(np.logical_and(ZSL99run2<=zmaxAve,ZSL99run2>=zr))    

ScSL99run1O = np.nanmean(ScSL99run1[rangeSL99run1])
ScSL99run2O = np.nanmean(ScSL99run2[rangeSL99run2])

ScSL99run1N = np.nanmean(ScFluxSL99run1[rangeSL99run1])
ScSL99run2N = np.nanmean(ScFluxSL99run2[rangeSL99run2])

CwSL99run1N = np.nanmean(CwSL99run1[rangeSL99run1])
CwSL99run2N = np.nanmean(CwSL99run2[rangeSL99run2])

pCwrun1, success = optimize.leastsq(errfunc, pinit[:], args=(ZSL99run1[rangeSL99run1], CwSL99run1[rangeSL99run1]))
pCwrun2, success = optimize.leastsq(errfunc, pinit[:], args=(ZSL99run2[rangeSL99run2], CwSL99run2[rangeSL99run2]))


print ('ScSL99run1 Old=',ScSL99run1O)
print ('ScSL99run2 Old=',ScSL99run2O)

print ('ScSL99run1 New=',ScSL99run1N)
print ('ScSL99run2 New=',ScSL99run2N)

print ('CwSL99run1N=',CwSL99run1N)
print ('CwSL99run2N=',CwSL99run2N)
#
# Plot parameters
#
zmin =-0.03;
zmax = 0.3;

nuf=1e-6

#S1 : s=1.2 dp=3e-3

taup=0.0715

#eps = (u_star_shear*np.exp(-1.34*Z_exp/H_exp))**3/3e-3
eps = (wrms_exp)**3/3e-3

tau_eta = (nuf/eps)**(1./2.)
tau_v = nuf/u_star_shear**2.
eta = (nuf**3/eps)**(1./4.)
tau_L = H_exp**(2./3.)/eps**(1./3.)

dp_eta_exp = 3e-3 / eta
St_eta_exp = taup / tau_eta
St_v_exp = taup / tau_v
St_L_exp = taup / tau_L


#eps = (u_star_shear*np.exp(-1.34*(Z_LES-hb_LES)/H_LES))**3/3e-3
eps = (wrms_LES)**3/3e-3

tau_eta = (nuf/eps)**(1./2.)
tau_v = nuf/u_star_shear**2.
eta = (nuf**3/eps)**(1./4.)
tau_L = H_LES**(2./3.)/eps**(1./3.)

dp_eta_LES = 3e-3 / eta
St_eta_LES = taup / tau_eta
St_v_LES = taup / tau_v
St_L_LES = taup / tau_L
#S2 : s=2.6 dp=135e-6
#-----------------------------
#s=rhop/rhof	d (microns)	Theta	S=Ws/u*	Stη	d/η	Rep	G l*/dp
#2.6 135.0 1.1798240646354818 0.21664221177640533 0.9720356884472436 4.187721865959449 1.4623349294907362 6.214341960336589 0.22884713829208359
#u*= 0.05 m/s
#-----------------------------
#s=rhop/rhof	d	u*	Ws	taup	taueta eta l*
#2.6 135.0 0.05 0.010832110588820268 0.0010101691194964906 0.0010392304845413263 3.223709795470625e-05 3.0894363669431286e-05
#-----------------------------
taup=0.001
eps = (WrmsSL99run1)**3/dSL99run1
tau_eta = (nuf/eps)**(1./2.)
tau_v = nuf/ufricSL99run1**2.
eta = (nuf**3/eps)**(1./4.)
tau_L = HSL99run1**(2./3.)/eps**(1./3.)

dp_eta_run1 = dSL99run1 / eta
St_eta_run1 = taup / tau_eta
St_v_run1 = taup / tau_v
St_L_run1 = taup / tau_L

eps = (WrmsSL99run2)**3/dSL99run2
tau_eta = (nuf/eps)**(1./2.)
tau_v = nuf/ufricSL99run2**2.
eta = (nuf**3/eps)**(1./4.)
tau_L = HSL99run2**(2./3.)/eps**(1./3.)


dp_eta_run2 = dSL99run2 / eta
St_eta_run2 = taup / tau_eta
St_v_run2 = taup / tau_v
St_L_run2 = taup / tau_L
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure1
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[0]==1):

    figure(num=1, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
    
    ax1 = subplot(gs[0, 0])
    ax1.plot(U_exp,Z_exp/H_exp,'+k')
    ax1.plot(U_LES,(Z_LES-hb_LES)/H_LES,'b-');
    ax1.set_xlim([0, 0.85])
    ax1.set_ylim([zmin, zmax])
    ax1.set_ylabel(r'$z/H_f$')
    ax1.set_xlabel(r'$<u>$ (m/s)')
    ax1.set_title('(a)')
    #ax1.grid()
    #
    #% inset logscale
    #
    axin1 = ax1.inset_axes([.25, 0.635, .425, .35]) #left bottom width height
    axin1.semilogy(U_exp,Z_exp/H_exp,'+k')
    axin1.semilogy(U_LES,(Z_LES-hb_LES)/H_exp,'b-')
    axin1.set_xlim(0, 1)
    axin1.set_ylim(1e-2, zmax)
    y_ticks = np.array([1e-2, 1e-1])
    y_labels = ['0.01','0.1']
    axin1.set_yticks(y_ticks)
    axin1.set_yticklabels(y_labels)
    #axin1.grid()     
    
    ax2 = subplot(gs[0, 1])
    ax2.plot(phi_exp,Z_exp/H_exp,'+k')
    ax2.plot(phi_LES,(Z_LES-hb_LES)/H_LES,'b-')
    ax2.set_xlim([0, 0.65])
    ax2.set_ylim([zmin, zmax])
    ax2.set_xlabel(r'$\phi$')
    ax2.set_title('(b)')
    ax2.set_yticklabels([])
    #ax2.grid()
    #
    #% inset logscale
    #
    axin2 = ax2.inset_axes([.295, 0.58, .65, .4]) #left bottom width height
    axin2.semilogx(phi_exp,Z_exp/H_exp,'+k')
    axin2.semilogx(phi_LES,(Z_LES-hb_LES)/H_exp,'b-')
    axin2.set_xlim(1e-5, 1)
    axin2.set_ylim(zmin, zmax)
    #axin2.grid()     
    
    ax3 = subplot(gs[0, 2])
    ax3.plot(-uw_exp,Z_exp/H_exp,'+k')
    ax3.plot(uw_LES,(Z_LES-hb_LES)/H_LES,'-b')
    ax3.set_xlim([0, 0.0029])
    ax3.set_ylim([zmin, zmax])
    ax3.set_title('(c)')
    x_ticks = np.array([0, 1e-3,2e-3])
    x_labels = ['0','0.001','0.002']
    ax3.set_xticks(x_ticks)
    ax3.set_xticklabels(x_labels)
    ax3.set_yticklabels([])
    ax3.set_xlabel(r'$<u^\prime w^\prime> (m^2/s^2)$')
    #ax3.grid()
    
    
    ax4 = subplot(gs[0, 3])
    ax4.plot(wphi_exp,Z_exp/H_exp,'+k')
    ax4.plot(wphi_LES,(Z_LES-hb_LES)/H_LES,'b-')
    ax4.set_xlim([0, 0.00175])
    ax4.set_ylim([zmin, zmax])
    ax4.set_title('(d)')
    x_ticks = np.array([0, 1e-3])
    x_labels = ['0','0.001']
    ax4.set_xticks(x_ticks)
    ax4.set_xticklabels(x_labels)
    ax4.set_yticklabels([])
    #set(gca,'XTick',[0 1e-3 2e-3],'XTicklabel',{'0', '0.001', '0.002'})
    ax4.set_xlabel(r'$<w^\prime\phi^\prime>$ (m/s)')
    #ax4.grid()
    #
    #% inset logscale
    #
    axin4 = ax4.inset_axes([.295, 0.58, .65, .4]) #left bottom width height
    axin4.semilogx(wphi_exp,Z_exp/H_exp,'+k')
    axin4.semilogx(wphi_LES,(Z_LES-hb_LES)/H_exp,'b-')
    axin4.set_xlim(1e-7, 3e-3)
    axin4.set_ylim(zmin, zmax)
    #set(gca,'YTicklabel',[], 'Fontsize', 14)
    #axin4.grid()
    
    #    
    fig1name = './figures/U_phi_fluxWU_NoSubgrid';
    
    savefig(fig1name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig1name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure2
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[1]==1):
    figure(num=2, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
    
    ax1 = subplot(gs[0, 0])
    ax1.plot(lm_exp/H_exp,Z_exp/H_exp,'+k')
    ax1.plot(lm_LES/H_LES,(Z_LES-hb_LES)/H_LES,'b-');
    ax1.plot(kappa_exp*(Z_exp-zdexp)/H_exp,Z_exp/H_exp,'--k')
    #ax1.plot([0.012, 0.012+0.1*0.225],[0.05, 0.15],'k-');
    #ax1.plot([0.012, 0.012+0.1*0.225],[0.05, 0.05],'k-');
    #ax1.plot([0.012+0.1*0.225, 0.012+0.1*0.225],[0.05, 0.15],'k-');
    ax1.set_xlim([0, 0.075])
    ax1.set_ylim([zmin, zmax])
    x_ticks = np.array([0, 0.025,0.05])
    x_labels = ['0','0.025','0.05']
    ax1.set_xticks(x_ticks)
    ax1.set_xticklabels(x_labels)
    ax1.set_ylabel(r'$z/H_f$')
    ax1.set_xlabel(r'$l_m/H_f$')
    ax1.set_title('(a)')
    #ax1.grid()
    
    ax2 = subplot(gs[0, 1])
    ax2.plot((nut_exp*np.abs(gradphi_exp))/(phi_exp*wfall),Z_exp/H_exp,'+k')
    ax2.plot((nut_LES*np.abs(gradphi_LES))/(phi_LES*wfall),(Z_LES-hb_LES)/H_LES,'b-');
    ax2.plot([0.33, 0.33],[zmin, 1],'k:');
    ax2.set_xlim([0, 1.2])
    ax2.set_ylim([zmin, zmax])
    ax2.set_xlabel(r'$S_{w\phi}=\nu^t \frac{d<\phi>}{dz} / w_s^0 <\phi>$')
    ax2.set_title('(b)')
    ax2.set_yticklabels([])
    #ax2.grid()
    
    ax3 = subplot(gs[0, 2])
    ax3.plot((nut_exp*np.abs(gradphi_exp))/wphi_exp,Z_exp/H_exp,'+k')
    ax3.plot((nut_LES*np.abs(gradphi_LES))/wphi_LES,(Z_LES-hb_LES)/H_LES,'b-');
    #ax3.plot([1, 1],[zmin, 1],'k:');
    ax3.set_xlim([0, 17])
    ax3.set_ylim([zmin, zmax])
    ax3.set_title('(c)')
    ax3.set_yticklabels([])
    ax3.set_xlabel(r'$S_{w^\prime\phi^\prime}= \nu^t \frac{d<\phi>}{dz} / <w^\prime\phi^\prime>$')
    #ax3.grid()
    
    ax4 = subplot(gs[0, 3])
    ax4.plot(wphi_exp/(phi_exp*wfall),Z_exp/H_exp,'+k')
    ax4.plot(wphi_LES/(phi_LES*wfall),(Z_LES-hb_LES)/H_LES,'b-')
    
    ax4.plot([0.16, 0.16],[zmin, 1],'k:');    
    ax4.set_xlim([0, 1.])
    ax4.set_ylim([zmin, zmax])
    ax4.set_title('(d)')
    ax4.set_yticklabels([])
    ax4.set_xlabel(r'$C_w=<w^\prime\phi^\prime>/w^0_s<\phi>$') 
    #ax4.grid()
    
    fig2name = './figures/wallnormalflux_NoSubgrid';
    
    savefig(fig2name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig2name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure3
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[2]==1):
    figure(num=3, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
    
    ax0 = subplot(gs[0, 0])
    ax0.plot(U_exp,Z_exp/H_exp,'+k')
    ax0.plot(USL99run1[:],ZSL99run1[:],'dg');
    ax0.plot(USL99run2[:],ZSL99run2[:],'sr');
    ax0.set_xlim([0, 0.9])
    ax0.set_ylim([zmin, zmax])
    ax0.set_ylabel(r'$z/H_f$')
    ax0.set_xlabel(r'$U (m/s)$')
    ax0.set_title('(a)')
    #ax0.grid()
    
    ax1 = subplot(gs[0, 1])
    ax1.plot(phi_exp,Z_exp/H_exp,'xk')
    ax1.plot(phiSL99run1[:],ZSL99run1[:],'dg');
    ax1.plot(phiSL99run2[:],ZSL99run2[:],'or');
    ax1.set_xscale('log')
    ax1.set_xlim([1e-4, 2])
    ax1.set_ylim([zmin, zmax])
    ax1.set_yticklabels([])
    ax1.set_xlabel(r'$\phi$')
    ax1.set_title('(b)')
    #ax1.grid()
    
    ax2 = subplot(gs[0, 2])
    ax2.plot(-uw_exp,Z_exp/H_exp,'+k')
    ax2.plot(uwSL99run1[:]/1000,ZSL99run1[:],'dg')
    ax2.plot(uwSL99run2[:]/1000,ZSL99run2[:],'or')
    ax2.set_xlim([0, 0.0029])
    ax2.set_ylim([zmin, zmax])
    ax2.set_xlabel(r'$<u^\prime w^\prime> (m^2/s^2)$')
    ax2.set_title('(c)')
    x_ticks = np.array([0, 1e-3,2e-3])
    x_labels = ['0','0.001','0.002']
    ax2.set_xticks(x_ticks)
    ax2.set_xticklabels(x_labels)
    ax2.set_yticklabels([])
    #ax2.grid()
    
    ax3 = subplot(gs[0, 3])
    ax3.plot(wphi_exp,Z_exp/H_exp,'+k')
    ax3.plot(CpWpSL99run1[:]/rho_sand,ZSL99run1[:],'dg')   
    ax3.plot(CpWpSL99run2[:]/rho_sand,ZSL99run2,'or')   
    ax3.set_xlim([1e-6, 0.001])
    ax3.set_ylim([zmin, zmax])
    ax3.set_xscale('log')
    ax3.set_title('(d)')
    ax3.set_yticklabels([])
    ax3.set_xlabel(r'$<w^\prime\phi^\prime>$ (m/s)')
    #ax3.grid()
        
    fig3name = './figures/Shencomparison';
    
    savefig(fig3name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig3name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure5
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[3]==1):
    figure(num=4, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
       
    ax1 = subplot(gs[0, 0])
    ax1.plot(lm_exp/H_exp,Z_exp/H_exp,'+k')
    ax1.plot(lm_SL99run1[:]/HSL99run1,ZSL99run1[:-1],'dg')    
    ax1.plot(lm_SL99run2[:]/HSL99run2,ZSL99run2[:-1],'or') 
    ax1.plot(0.4*(ZSL99run1-zdSL99run1/HSL99run1),ZSL99run1,'--g')    
    ax1.plot(0.4*(ZSL99run2-zdSL99run2/HSL99run2),ZSL99run2,'--r')   
    ax1.set_xlim([0, 0.125])
    ax1.set_ylim([zmin, zmax])
    ax1.set_ylabel(r'$z/H_f$')
    ax1.set_xlabel(r'$l_m/H_f$')
    ax1.set_title('(a)')
    #ax1.grid()
    
    ax2 = subplot(gs[0, 1])
    ax2.plot((nut_exp*np.abs(gradphi_exp))/(phi_exp*wfall),Z_exp/H_exp,'+k')
    ax2.plot(ScSL99run1,ZSL99run1[:-1],'dg')
    ax2.plot(ScSL99run2,ZSL99run2[:-1],'or')
    ax2.plot([0.33, 0.33],[zmin, 1],'k:');
    ax2.set_xlim([0, 2.2])
    ax2.set_ylim([zmin, zmax])
    ax2.set_xlabel(r'$S_{w\phi}=\nu^t \frac{d<\phi>}{dz} / w_s^0 <\phi>$')
    ax2.set_title('(b)')
    ax2.set_yticklabels([])
    #ax2.grid()
    
    ax3 = subplot(gs[0, 2])
    ax3.plot((nut_exp*np.abs(gradphi_exp))/wphi_exp,Z_exp/H_exp,'+k')
    ax3.plot(ScFluxSL99run1,ZSL99run1[:-1],'dg')
    ax3.plot(ScFluxSL99run2,ZSL99run2[:-1],'or')   
    ax3.set_xlim([0, 7])
    ax3.set_ylim([zmin, zmax])
    ax3.set_title('(c)')
    ax3.set_yticklabels([])
    ax3.set_xlabel(r'$S_{w^\prime\phi^\prime}= \nu^t \frac{d<\phi>}{dz} / <w^\prime\phi^\prime>$')
    #ax3.grid()
    
    ax4 = subplot(gs[0, 3])
    ax4.plot(wphi_exp/(phi_exp*wfall),Z_exp/H_exp,'+k')
    ax4.plot(CwSL99run1,ZSL99run1,'dg');
    ax4.plot(CwSL99run2,ZSL99run2,'or');
    ax4.plot(fitfunc(pCwrun1,ZSL99run1[:] ),ZSL99run1,'--g')
    ax4.plot(fitfunc(pCwrun2,ZSL99run2[:] ),ZSL99run2,'--r')
    ax4.plot([0.16, 0.16],[zmin, 1],'k:');    
    ax4.set_xlim([0, 1.])
    ax4.set_ylim([zmin, zmax])
    ax4.set_title('(d)')
    ax4.set_yticklabels([])
    ax4.set_xlabel(r'$C_w=<w^\prime\phi^\prime>/w^0_s<\phi>$') 
    #ax4.grid()
    
    fig4name = './figures/Shencomparison2';
    
    savefig(fig4name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig4name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')
    
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure5
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[4]==1):
    figure(num=5, figsize=(figwidth, figheight),
            dpi=60, facecolor='w', edgecolor='w')
    nu=1e-6
    Z_expPlus = (Z_exp-zdexp) * u_star_shear/nu
    gradUExpPlus = gradUExp*nu/u_star_shear**2
    
    Z_LESPlus = (Z_LES-hb_LES-zdexp)*u_star_shear/nu
    gradULESPlus = gradULES*nu/u_star_shear**2
    
    ReTau = H_exp*u_star_shear/nu
    print('ReTau=',ReTau)
    print((0.275*H_exp-zdexp) * u_star_shear/nu)
    print((0.05*H_exp-zdexp) * u_star_shear/nu)
    
    ax1 = subplot(111)
    #ax1.set_title(r'$Re_{\tau}=8500$')
    ax1.semilogx(Z_expPlus,Z_expPlus*gradUExpPlus,'+k')
    ax1.plot(Z_LESPlus,Z_LESPlus*gradULESPlus,'b-')
    ax1.plot([3*np.sqrt(ReTau), 3*np.sqrt(ReTau)],[0,20],':k')
    ax1.plot([0.15*ReTau, 0.15*ReTau],[0,20],':k')
    ax1.plot([1, 1e4],[1/kappa_exp,1/kappa_exp],'--r')
    ax1.plot([1, 1e4],[1/0.41,1/0.41],'-.k')
    
    ax1.set_ylim([0, 6.5])
    ax1.set_xlim([10, 0.65*H_exp*u_star_shear/nu])
    ax1.set_xlabel(r'$z^+$')
    ax1.set_ylabel(r'$z^+ dU^+/dz^+$')
    #ax1.grid()
    
    fig5name = './figures/loglayer';
    
    savefig(fig5name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig5name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure6
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[5]==1):
    figure(num=6, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
    
# Compute mean quantities from exp dataShen
    CwSL99run1O = 1
    #ScSL99run1O = 1./0.74 # from Shen
    Rorun1Old = CwSL99run1O*ScSL99run1O*WsSL99run1/(0.4*ufricSL99run1)
    Rouserun1Old = phiSL99run1[1]*((1-ZSL99run1)/ZSL99run1*(ZSL99run1[1]/(1-ZSL99run1[1])))**(Rorun1Old)
    
    CwSL99run2O = 1
    #ScSL99run2O = 1./0.82 # from Shen
    Rorun2Old = CwSL99run2O*ScSL99run2O*WsSL99run2/(0.4*ufricSL99run2)
    ZSL99run2_2=ZSL99run2#-zdSL99run2/HSL99run2
    Rouserun2Old = phiSL99run2[0]*((1-ZSL99run2)/ZSL99run2*(ZSL99run2[0]/(1-ZSL99run2[0])))**(Rorun2Old)
     
    #Rorun1New = CwSL99run1N*ScSL99run1N*WsSL99run1/(0.4*ufricSL99run1)
    # Integrated Rouse with settling retardation and Sc>1
    ScSL99run1N=ScexpN
    irefrun1=np.min(np.where(ZSL99run1>=zr))
    Rouserun1New = np.zeros(np.size(ZSL99run1))
    Rouserun1New[:] = np.nan
    dum = 0.0
    Rouserun1New[irefrun1] = phiSL99run1[irefrun1]
    for i in range(np.size(np.where(ZSL99run1>=zr))-1):
        dum = dum + fitfunc(pCwrun1,ZSL99run1[i+irefrun1+1] )*ScSL99run1N*WsSL99run1/\
            (0.4**2*(ZSL99run1[irefrun1+i+1]*HSL99run1-zdSL99run1)**2\
             *gradUrun1[i+irefrun1+1])*(ZSL99run1[i+irefrun1+1]-ZSL99run1[i+irefrun1])*HSL99run1
        Rouserun1New[i+irefrun1+1] = phiSL99run1[irefrun1]*np.exp(-dum)
        #print(iref135,i,dum,C135[iref135,0]/rho_sand,Rouse135New[iref135+i+1])

    ScSL99run2N=ScexpN
    irefrun2=np.min(np.where(ZSL99run2>=zr))
    Rouserun2New = np.zeros(np.size(ZSL99run2))
    Rouserun2New[:] = np.nan
    dum = 0.0
    Rouserun2New[irefrun2] = phiSL99run2[irefrun2]
    for i in range(np.size(np.where(ZSL99run2>=zr))-1):
        dum = dum + fitfunc(pCwrun2,ZSL99run2[i+irefrun2+1] )*ScSL99run2N*WsSL99run2/\
            (0.4**2*(ZSL99run2[irefrun2+i+1]*HSL99run2-zdSL99run2)**2\
             *gradUrun2[i+irefrun2+1])*(ZSL99run2[i+irefrun2+1]-ZSL99run2[i+irefrun2])*HSL99run2
        Rouserun2New[i+irefrun2+1] = phiSL99run2[irefrun2]*np.exp(-dum)
    ######
    
    Roexp=Scexp*wfall/(0.41*u_star_shear)
    iref=np.max(np.where(Z_exp2>=0))
    Rouse_exp=phi_exp[iref]*((1-Z_exp2)/Z_exp2*(Z_exp2[iref]/(1-Z_exp2[iref])))**(Roexp)
    Rouse_exp[np.where(Z_exp2<=0)] = np.nan
    
# Integrated Rouse with settling retardation and Sc>1
    CwexpN = 0.12
    Rouse_expNew = np.zeros(Nexp)
    Rouse_expNew[:] = np.nan
    dum = 0.0
    Rouse_expNew[iref] = phi_exp[iref]
    for i in range(iref):
        dum = dum + ScexpN*CwexpN*wfall/(kappa_exp**2*(Z_exp[iref-i]-zdexp)**2*gradUExp[iref-i])*(Z_exp[iref-i-1]-Z_exp[iref-i])
        Rouse_expNew[iref-i-1] = phi_exp[iref]*np.exp(-dum)
        #print(iref,i,dum,phi_exp[iref],Rouse_expNew[iref-i-1])

# Integrated Rouse with settling velocity in quiescent water and Sc<1
    Rouse_expOld = np.zeros(Nexp)
    Rouse_expOld[:] = np.nan
    dum = 0.0
    Rouse_expOld[iref] = phi_exp[iref]
    for i in range(iref):
        dum = dum + Scexp*wfall/(0.41**2*(Z_exp[iref-i]-zdexp)**2*gradUExp[iref-i])*(Z_exp[iref-i-1]-Z_exp[iref-i])
        Rouse_expOld[iref-i-1] = phi_exp[iref]*np.exp(-dum)
        
# Integrated Rouse with settling velocity in quiescent water and Sc<1
    Rouse_expOld2 = np.zeros(Nexp)
    Rouse_expOld2[:] = np.nan
    dum = 0.0
    Rouse_expOld2[iref] = phi_exp[iref]
    for i in range(iref):
        dum = dum + Scexp*wfall/(kappa_exp**2*(Z_exp[iref-i]-zdexp)**2*gradUExp[iref-i])*(Z_exp[iref-i-1]-Z_exp[iref-i])
        Rouse_expOld2[iref-i-1] = phi_exp[iref]*np.exp(-dum)
        
    ax0 = subplot(111)
    ax0.plot(phi_exp,Z_exp/H_exp,'xk',label='RB15')
    ax0.plot(phiSL99run1[:],ZSL99run1[:],'dg',label='SL99_run1');
    ax0.plot(phiSL99run2[:],ZSL99run2[:],'or',label='SL99_run2');
    #ax0.plot(Rouse_exp,Z_exp/H_exp,':k',label='Rouse (Eq. 1)');
    ax0.plot(Rouse_expOld,Z_exp/H_exp,':k',lw=4,label=r'Rouse Orig. $\kappa$=0.41');
    ax0.plot(Rouse_expOld2,Z_exp/H_exp,'-.k',lw=4,label=r'Rouse Orig. $\kappa$=0.22');
    ax0.plot(Rouse_expNew,Z_exp/H_exp,'--k',lw=4,label='Rouse New');
    
    ax0.plot(Rouserun1Old,ZSL99run1,':g',lw=4,label='Rouse Orig.');
    ax0.plot(Rouserun2Old,ZSL99run2,':r',lw=4,label='Rouse Orig.');
    ax0.plot(Rouserun1New,ZSL99run1,'--g',lw=4,label='Rouse New');
    ax0.plot(Rouserun2New,ZSL99run2,'--r',lw=4,label='Rouse New');
    
    
    
    ax0.legend(fontsize=fontsize*0.78)
    ax0.set_xscale('log')
    ax0.set_xlim([2e-4, 0.3])
    ax0.set_ylim([0.1, zmax])
    ax0.set_ylabel(r'$z/H_f$')
    ax0.set_xlabel(r'$\phi$')
    #ax0.grid()
    
    
    fig6name = './figures/ConcentrationProfile';
    
    savefig(fig6name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig6name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')
    
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#% Plot figure7
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (figures2plot[6]==1):
    figure(num=7, figsize=(figwidth, figheight),
           dpi=60, facecolor='w', edgecolor='w')
 
    phi=np.linspace(0,0.6,1000)
    wrms=np.linspace(0,2,100)

    ax0 = subplot(gs2[0, 0])
    ax0.set_title('(a)')
    ax0.plot(phi_exp,wphi_exp/(phi_exp*wfall),'+k')
    ax0.plot(phiSL99run1,CwSL99run1,'dg')
    ax0.plot(phiSL99run2,CwSL99run2,'or') 
    ax0.plot(phi,(1-phi)**4.65,'--k')
    ax0.set_xscale('log')
    ax0.set_xlim([1e-5, 0.6])
    ax0.set_ylim([0, 1])
    ax0.set_ylabel(r'$Cw$')
    ax0.set_xlabel(r'$\phi$')
    #ax0.grid()

    
    ax1 = subplot(gs2[0, 1])
    ax1.set_title('(b)')
    ax1.plot(wrms_exp[range_exp]/wfall,wphi_exp[range_exp]/(phi_exp[range_exp]*wfall),'+k')
    ax1.plot(WrmsSL99run1[rangeSL99run1]/WsSL99run1,CwSL99run1[rangeSL99run1],'dg')
    ax1.plot(WrmsSL99run2[rangeSL99run2]/WsSL99run1,CwSL99run2[rangeSL99run2],'or') 
    ax1.plot(wrms,1-1.3*wrms,'--k')
    ax1.set_xlim([0, 3])
    ax1.set_ylim([0., 1.])
    ax1.set_yticklabels([])
    ax1.set_xlabel(r'$w_{rms}/W^0_s$')
    #ax1.grid()
    
    ax2 = subplot(gs2[0, 2])
    ax2.set_title('(c)')
    ax2.plot(St_eta_exp[range_exp],wphi_exp[range_exp]/(phi_exp[range_exp]*wfall),'+k')
    ax2.plot(St_eta_run1[rangeSL99run1],CwSL99run1[rangeSL99run1],'dg')
    ax2.plot(St_eta_run2[rangeSL99run2],CwSL99run2[rangeSL99run2],'or') 
    ax2.set_xscale('log')
    ax2.set_ylim([0., 1.])
    ax2.set_yticklabels([])
    ax2.set_xlabel(r'$St_\eta$')
    #ax2.grid()
    
    fig7name = './figures/SettlingRetardation';
    
    savefig(fig7name+'.eps', dpi=300, facecolor='w', edgecolor='w',format='eps')
    savefig(fig7name+'.png', dpi=300, facecolor='w', edgecolor='w',format='png')